@@ -60,13 +60,9 @@ typedef enum
 
 class __EXPORT process
 {
-private:
-	static errlevel_t verbose;
-
 public:
-	inline static void setVerbose(errlevel_t idx)
-		{verbose = idx;};
-
+	static void setVerbose(errlevel_t idx);
+	static void setHistory(unsigned limit);
 	static const char *identity(void);
 	static void printlog(const char *fmt, ...) __PRINTF(1, 2);
 	static void errlog(errlevel_t log, const char *fmt, ...) __PRINTF(2, 3);
@@ -78,6 +74,8 @@ public:
 	static void release(void);
 	static bool state(const char *value);
 	static bool system(const char *fmt, ...) __PRINTF(1, 2);
+	static void histlog(const char *uid);
+	static void siplog(const char *uid);
 	static FILE *dumpfile(const char *uid);
 	static FILE *snapshot(const char *uid);
 	static FILE *statfile(void);
