@@ -1096,6 +1096,11 @@ void server::run(const char *user)
 			continue;
 		}
 
+		if(!stricmp(cp, "siplog")) {
+			service::siplog(user);
+			continue;
+		}
+
 		if(!stricmp(cp, "dump")) {
 			service::dumpfile(user);
 			continue;
@@ -1115,8 +1120,32 @@ void server::run(const char *user)
 		if(argc < 1)
 			continue;
 
+		if(!stricmp(argv[0], "history")) {
+			if(argc > 2)
+				goto invalid;
+			else if(argc == 2)
+				process::setHistory(atoi(argv[1]));
+			else
+				service::history(user);
+			continue;
+		}
+
+		if(!stricmp(argv[0], "trace")) {
+			if(argc != 2)
+				goto invalid;
+			if(!stricmp(argv[1], "on"))
+				stack::enableDumping();
+			else if(!stricmp(argv[1], "off"))
+				stack::disableDumping();
+			else if(!stricmp(argv[1], "clear"))
+				stack::clearDumping();
+			else
+				goto invalid;
+			continue;
+		}
+
 		if(!stricmp(argv[0], "verbose")) {
-			if(argc > 2) {
+			if(argc != 2) {
 invalid:
 				process::reply("invalid argument");
 				continue;
